#version 120
// build grain plate
uniform sampler2D adsk_results_pass1;
uniform float adsk_result_w, adsk_result_h, adsk_time, adsk_result_pixelratio, adsk_result_frameratio;
uniform bool animated_noise;
uniform vec4 gain, scale;

float hash( float n ) {
    return fract(sin(n)*687.3123);
}

float noise( in vec2 x ) {
    vec2 p = floor(x);
    vec2 f = fract(x);
    f = f*f*(3.0-2.0*f);
    float n = p.x + p.y*157.0;
    return mix(mix( hash(n+  0.0), hash(n+  1.0),f.x),
               mix( hash(n+157.0), hash(n+158.0),f.x),f.y);
}

const mat2 m2 = mat2( 0.80, -0.60, 0.60, 0.80 );

float fbm( vec2 p ) {
    float f = 0.0;
    f += 0.5000*noise( p ); p = m2*p*2.02;
    f += 0.2500*noise( p ); p = m2*p*2.03;
    f += 0.1250*noise( p ); p = m2*p*2.01;
		f += 0.0625*noise( p );

    return f/0.9375;
}

void main(void)
{
	vec2 uv = gl_FragCoord.xy / vec2( adsk_result_w, adsk_result_h);
  float time = adsk_time;
  if ( animated_noise )
    time = 0.0;
  vec4 c = vec4(0.0);
	vec2 off_center = vec2(0.0);

	// random x y
	off_center.x = noise(vec2((time + 134.) * 148.12, (time + 115.) * 13.56));
	off_center.y = noise(vec2((time + 54.) * 151.1, (time + 135.) * 84.3));

	float rnd = 0.0;
 	// random rotation
	rnd = noise(vec2((time + 67.) * 123.32)) * 214.2;
	rnd += noise(vec2((time + 42.) * 34.1)) * 213.53;

	float rot_cent = noise(vec2((time + 5678.) * 123.32)) * 214.2;
  mat2 rot = mat2( cos(rnd - rot_cent), -sin( rnd - rot_cent), sin( rnd - rot_cent), cos( rnd - rot_cent));

  vec2 r_uv = uv;
  r_uv -= vec2(0.5);
  r_uv.x *= adsk_result_frameratio;
  r_uv *= rot;
  r_uv /= scale.y * scale.x * 0.4;
  r_uv += vec2(0.5);

  vec2 g_uv = uv;
  g_uv -= vec2(0.5);
  g_uv.x *= adsk_result_frameratio;
  g_uv *= rot;
  g_uv /= scale.z * scale.x * 0.4;
  g_uv += vec2(0.5);

  vec2 b_uv = uv;
  b_uv -= vec2(0.5);
  b_uv.x *= adsk_result_frameratio;
  b_uv *= rot;
  b_uv /= scale.a * scale.x * 0.4;
  b_uv += vec2(0.5);

  float r_g = texture2D(adsk_results_pass1, (r_uv + off_center)).r;
  float g_g = texture2D(adsk_results_pass1, (g_uv + off_center)).g;
  float b_g = texture2D(adsk_results_pass1, (b_uv + off_center)).b;
  vec3 g = vec3(r_g, g_g, b_g);

	// gain red noise channel
	g.r *= gain.y;
	// gain green noise channel
	g.g *= gain.z;
	// gain blue noise channel
	g.b *= gain.a;
	// overall gain
	g *= gain.x;


	gl_FragColor.rgb = g;
}
