#version 120
uniform sampler2D source, matte, adsk_results_pass5;
uniform float adsk_result_w, adsk_result_h, adsk_result_pixelratio, adsk_result_frameratio;
uniform bool show_grain;

// Forward declaration of API function. This is necessary to use in Matchbox otherwise it won't compiled. Please see MatchboxAPI for more info.
float adsk_getLuminance ( vec3 rgb );
float adskEvalDynCurves( int curve, float x );

// Here is the int used for the single Luma Curve widget
uniform ivec4 lumaCurve;

void main(void)
{
	vec2 uv = gl_FragCoord.xy / vec2( adsk_result_w, adsk_result_h);
  vec4 c = vec4(0.0);

	vec3 s = texture2D(source, uv).rgb;
	vec3 g = texture2D(adsk_results_pass5, uv).rgb;

	float m = texture2D(matte, uv).r;

  // Extract Luminance from source using API function
  float lum = adsk_getLuminance(s);
  // Here I'm evluating the RGB Master Luma curve widget
  float master_lum = adskEvalDynCurves(lumaCurve.r, lum);
  float red_lum = adskEvalDynCurves(lumaCurve.g, lum);
  float green_lum = adskEvalDynCurves(lumaCurve.b, lum);
  float blue_lum = adskEvalDynCurves(lumaCurve.a, lum);

	// master channel
	c.rgb = g + s;
	c.rgb = vec3(master_lum * c.rgb + (1.0 - master_lum) * s);
	c.rgb = vec3(m * c.rgb + (1.0 - m) * s);

	if ( show_grain )
		c = vec4(g + 0.5, 1.0);

	gl_FragColor = c;
}
